'use strict';

exports.__esModule = true;

exports.default = function (babel) {
  var t = babel.types;


  return {
    name: 'emotion', // not required
    inherits: require('babel-plugin-syntax-jsx'),
    visitor: {
      Program: {
        enter: function enter(path, state) {
          state.inline = path.hub.file.opts.filename === 'unknown' || state.opts.inline;
          state.staticRules = [];
          state.insertStaticRules = function (staticRules) {
            var _state$staticRules;

            (_state$staticRules = state.staticRules).push.apply(_state$staticRules, staticRules);
          };
        },
        exit: function exit(path, state) {
          if (state.staticRules.length !== 0) {
            var toWrite = state.staticRules.join('\n').trim();
            var filenameArr = path.hub.file.opts.filename.split('.');
            filenameArr.pop();
            filenameArr.push('emotion', 'css');
            var cssFilename = filenameArr.join('.');
            var exists = _fs2.default.existsSync(cssFilename);
            path.node.body.unshift(t.importDeclaration([], t.stringLiteral('./' + (0, _path.basename)(cssFilename))));
            if (exists ? _fs2.default.readFileSync(cssFilename, 'utf8') !== toWrite : true) {
              if (!exists) {
                (0, _touch.touchSync)(cssFilename);
              }
              _fs2.default.writeFileSync(cssFilename, toWrite);
            }
          }
        }
      },
      JSXOpeningElement: function JSXOpeningElement(path, state) {
        (0, _cssProp2.default)(path, t);
      },
      TaggedTemplateExpression: function TaggedTemplateExpression(path, state) {
        // in:
        // styled.h1`color:${color};`
        //
        // out:
        // styled('h1', "css-r1aqtk", [colorVar, heightVar], function inlineCss(x0, x1) {
        //   return [`.css-r1aqtk {
        //     margin: 12px;
        //     color: ${x0};
        //     height: ${x1}; }`];
        // });

        var parent = path.findParent(function (p) {
          return p.isVariableDeclarator();
        });
        var identifierName = parent && t.isIdentifier(parent.node.id) ? parent.node.id.name : '';

        function buildCallExpression(identifier, tag, path) {
          var built = (0, _attrs2.default)(path, t);

          var _inline = (0, _inline4.inline)(built, identifierName, 'css', state.inline

          // hash will be '0' when no styles are passed so we can just return the original tag
          ),
              hash = _inline.hash,
              rules = _inline.rules,
              name = _inline.name,
              hasOtherMatch = _inline.hasOtherMatch,
              hasApply = _inline.hasApply;

          if (hash === '0') {
            return tag;
          }
          var args = [tag, t.stringLiteral(name + '-' + hash), t.arrayExpression(built.expressions)];
          if (!hasApply && !hasOtherMatch && !state.inline) {
            state.insertStaticRules(rules);
          } else {
            var expressions = built.expressions.map(function (x, i) {
              return t.identifier('x' + i);
            });
            var inlineContentExpr = t.functionExpression(t.identifier('createEmotionStyledRules'), expressions, t.blockStatement([t.returnStatement(t.arrayExpression(parseDynamicValues(rules, t, expressions)))]));
            args.push(inlineContentExpr);
          }

          return t.callExpression(identifier, args);
        }

        if (
        // styled.h1`color:${color};`
        t.isMemberExpression(path.node.tag) && path.node.tag.object.name === 'styled' && t.isTemplateLiteral(path.node.quasi)) {
          path.replaceWith(buildCallExpression(path.node.tag.object, t.stringLiteral(path.node.tag.property.name), path));
        } else if (
        // styled('h1')`color:${color};`
        t.isCallExpression(path.node.tag) && path.node.tag.callee.name === 'styled' && t.isTemplateLiteral(path.node.quasi)) {
          path.replaceWith(buildCallExpression(path.node.tag.callee, path.node.tag.arguments[0], path));
        } else if (t.isIdentifier(path.node.tag) && path.node.tag.name === 'fragment') {
          var _inline2 = (0, _inline4.inline)(path.node.quasi, identifierName, 'frag', true),
              rules = _inline2.rules;

          if (rules.length > 1) {
            throw path.buildCodeFrameError('Fragments cannot have multiple selectors.');
          }
          var rulesWithoutSelector = rules.map(function (rule) {
            return rule.substring(rule.indexOf('{') + 1, rule.length - 1).trim();
          });
          path.replaceWith(parseDynamicValues(rulesWithoutSelector, t, path.node.quasi.expressions)[0]);
        } else if (t.isIdentifier(path.node.tag) && path.node.tag.name === 'css') {
          var _inline3 = (0, _inline4.inline)(path.node.quasi, identifierName, 'css', state.inline),
              hash = _inline3.hash,
              name = _inline3.name,
              _rules = _inline3.rules,
              hasApply = _inline3.hasApply,
              hasVar = _inline3.hasVar,
              hasOtherMatch = _inline3.hasOtherMatch;

          var classNameStringLiteral = t.stringLiteral(name + '-' + hash);
          var args = [classNameStringLiteral, t.arrayExpression(path.node.quasi.expressions)];
          if (!hasApply && !hasOtherMatch && !state.inline) {
            state.insertStaticRules(_rules);
            if (!hasVar) {
              return path.replaceWith(classNameStringLiteral);
            }
          } else {
            var expressions = path.node.quasi.expressions.map(function (x, i) {
              return t.identifier('x' + i);
            });
            var inlineContentExpr = t.functionExpression(t.identifier('createEmotionRules'), expressions, t.blockStatement([t.returnStatement(t.arrayExpression(parseDynamicValues(_rules, t, expressions)))]));
            args.push(inlineContentExpr);
          }
          path.replaceWith(t.callExpression(t.identifier('css'), args));
        } else if (t.isIdentifier(path.node.tag) && path.node.tag.name === 'keyframes') {
          var _keyframes = (0, _inline4.keyframes)(path.node.quasi, identifierName, 'animation'),
              _hash = _keyframes.hash,
              _name = _keyframes.name,
              _rules2 = _keyframes.rules,
              hasInterpolation = _keyframes.hasInterpolation;

          var animationName = _name + '-' + _hash;
          if (!hasInterpolation && !state.inline) {
            state.insertStaticRules(['@keyframes ' + animationName + ' ' + _rules2.join('')]);
            path.replaceWith(t.stringLiteral(animationName));
          } else {
            path.replaceWith(t.callExpression(t.identifier('keyframes'), [t.stringLiteral(animationName), t.arrayExpression(parseDynamicValues(_rules2, t, path.node.quasi.expressions))]));
          }
        } else if (t.isIdentifier(path.node.tag) && path.node.tag.name === 'fontFace') {
          var _fontFace = (0, _inline4.fontFace)(path.node.quasi, state.inline),
              _rules3 = _fontFace.rules,
              _hasInterpolation = _fontFace.hasInterpolation;

          if (!_hasInterpolation && !state.inline) {
            state.insertStaticRules(_rules3);
            if (t.isExpressionStatement(path.parent)) {
              path.parentPath.remove();
            } else {
              path.replaceWith(t.identifier('undefined'));
            }
          } else {
            path.replaceWith(t.callExpression(t.identifier('fontFace'), [t.arrayExpression(parseDynamicValues(_rules3, t, path.node.quasi.expressions))]));
          }
        } else if (t.isIdentifier(path.node.tag) && path.node.tag.name === 'injectGlobal' && t.isTemplateLiteral(path.node.quasi)) {
          var _injectGlobal = (0, _inline4.injectGlobal)(path.node.quasi),
              _rules4 = _injectGlobal.rules,
              _hasInterpolation2 = _injectGlobal.hasInterpolation;

          if (!_hasInterpolation2 && !state.inline) {
            state.insertStaticRules(_rules4);
            if (t.isExpressionStatement(path.parent)) {
              path.parentPath.remove();
            } else {
              path.replaceWith(t.identifier('undefined'));
            }
          } else {
            path.replaceWith(t.callExpression(t.identifier('injectGlobal'), [t.arrayExpression(parseDynamicValues(_rules4, t, path.node.quasi.expressions))]));
          }
        }
      }
    }
  };
};

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _touch = require('touch');

var _inline4 = require('./inline');

var _attrs = require('./attrs');

var _attrs2 = _interopRequireDefault(_attrs);

var _cssProp = require('./css-prop');

var _cssProp2 = _interopRequireDefault(_cssProp);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function parseDynamicValues(rules, t, inputExpressions) {
  return rules.map(function (rule) {
    var re = /xxx(\S)xxx/gm;
    var varMatch = void 0;
    var matches = [];
    while ((varMatch = re.exec(rule)) !== null) {
      matches.push({
        value: varMatch[0],
        p1: varMatch[1],
        index: varMatch.index
      });
    }

    var cursor = 0;

    var _matches$reduce = matches.reduce(function (accum, _ref, i) {
      var value = _ref.value,
          p1 = _ref.p1,
          index = _ref.index;
      var quasis = accum[0],
          expressions = accum[1];

      var preMatch = rule.substring(cursor, index);
      cursor = cursor + preMatch.length + value.length;
      if (preMatch) {
        quasis.push(t.templateElement({ raw: preMatch, cooked: preMatch }));
      }

      expressions.push(inputExpressions[p1]);

      if (i === matches.length - 1 && cursor <= rule.length) {
        var postMatch = rule.substring(cursor);

        quasis.push(t.templateElement({
          raw: postMatch,
          cooked: postMatch
        }, true));
      }
      return accum;
    }, [[], []]),
        quasis = _matches$reduce[0],
        expressions = _matches$reduce[1];

    if (!matches.length) {
      return t.templateLiteral([t.templateElement({ raw: rule, cooked: rule }, true)], []);
    }

    return t.templateLiteral(quasis, expressions);
  });
}