'use strict';

exports.__esModule = true;
exports.inline = inline;
exports.keyframes = keyframes;
exports.fontFace = fontFace;
exports.injectGlobal = injectGlobal;

var _parser = require('./parser');

var _hash = require('./hash');

var _hash2 = _interopRequireDefault(_hash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function extractNameFromProperty(str) {
  var regex = /name\s*:\s*([A-Za-z0-9\-_]+)\s*/gm;
  var match = regex.exec(str);
  if (match) {
    return match[1];
  }
}

function getName(extracted, identifierName, prefix) {
  var parts = [];
  parts.push(prefix);
  if (extracted) {
    parts.push(extracted);
  } else if (identifierName) {
    parts.push(identifierName);
  }
  return parts.join('-');
}

function replaceApplyWithPlaceholders(rules) {
  return rules.map(function (rule) {
    return rule.replace(/@apply\s+--[A-Za-z0-9-_]+-([0-9]+)/gm, function (match, p1) {
      return 'xxx' + p1 + 'xxx';
    });
  });
}

function createSrc(strs, name, hash) {
  var hasApply = false;
  var otherMatches = 0;
  var src = strs.reduce(function (arr, str, i) {
    arr.push(str);
    if (i !== strs.length - 1) {
      // todo - test for preceding @apply
      var applyMatch = /@apply\s*$/gm.exec(str);
      if (applyMatch) {
        hasApply = true;
        arr.push('--' + name + '-' + hash + '-' + i);
      } else {
        otherMatches++;
        arr.push('xxx' + i + 'xxx');
      }
    }
    return arr;
  }, []).join('').trim();
  return { src: src, hasApply: hasApply, otherMatches: otherMatches };
}

function inline(quasi, identifierName, prefix, inlineMode) {
  var strs = quasi.quasis.map(function (x) {
    return x.value.cooked;
  });
  var hash = (0, _hash2.default)([].concat(strs) // todo - add current filename?
  );var name = getName(extractNameFromProperty(strs.join('xxx')), identifierName, prefix);

  var _createSrc = createSrc(strs, name, hash),
      src = _createSrc.src,
      hasApply = _createSrc.hasApply,
      otherMatches = _createSrc.otherMatches;

  var _parseCSS = (0, _parser.parseCSS)('.' + name + '-' + hash + ' { ' + src + ' }', {
    inlineMode: inlineMode || hasApply,
    otherMatches: otherMatches,
    name: name,
    hash: hash
  }),
      rules = _parseCSS.rules,
      hasVar = _parseCSS.hasVar,
      hasOtherMatch = _parseCSS.hasOtherMatch;

  if (hasApply) {
    rules = replaceApplyWithPlaceholders(rules);
  }
  return { hash: hash, name: name, rules: rules, hasApply: hasApply, hasVar: hasVar, hasOtherMatch: hasOtherMatch };
}

function keyframes(quasi, identifierName, prefix) {
  var strs = quasi.quasis.map(function (x) {
    return x.value.cooked;
  });
  var hash = (0, _hash2.default)([].concat(strs));
  var name = getName(extractNameFromProperty(strs.join('xxx')), identifierName, prefix);

  var _createSrc2 = createSrc(strs, name, hash),
      src = _createSrc2.src,
      hasApply = _createSrc2.hasApply,
      otherMatches = _createSrc2.otherMatches;

  var _parseCSS2 = (0, _parser.parseCSS)('{ ' + src + ' }', {
    nested: false,
    inlineMode: true,
    otherMatches: otherMatches,
    name: name,
    hash: hash
  }),
      rules = _parseCSS2.rules,
      hasVar = _parseCSS2.hasVar,
      hasOtherMatch = _parseCSS2.hasOtherMatch;

  if (hasApply) {
    rules = replaceApplyWithPlaceholders(rules);
  }
  return { hash: hash, name: name, rules: rules, hasInterpolation: hasVar || hasApply || hasOtherMatch };
}

function fontFace(quasi) {
  var strs = quasi.quasis.map(function (x) {
    return x.value.cooked;
  });

  var _createSrc3 = createSrc(strs, 'name', 'hash'),
      src = _createSrc3.src,
      hasApply = _createSrc3.hasApply,
      otherMatches = _createSrc3.otherMatches;

  var _parseCSS3 = (0, _parser.parseCSS)('@font-face {' + src + '}', { otherMatches: otherMatches, inlineMode: true, name: 'name', hash: 'hash' }),
      rules = _parseCSS3.rules,
      hasVar = _parseCSS3.hasVar,
      hasOtherMatch = _parseCSS3.hasOtherMatch;

  if (hasApply) {
    rules = replaceApplyWithPlaceholders(rules);
  }
  return { rules: rules, hasInterpolation: hasVar || hasApply || hasOtherMatch };
}

function injectGlobal(quasi) {
  var strs = quasi.quasis.map(function (x) {
    return x.value.cooked;
  });

  var _createSrc4 = createSrc(strs, 'name', 'hash'),
      src = _createSrc4.src,
      otherMatches = _createSrc4.otherMatches,
      hasApply = _createSrc4.hasApply;

  var _parseCSS4 = (0, _parser.parseCSS)(src, { otherMatches: otherMatches, inlineMode: true, name: 'name', hash: 'hash' }),
      rules = _parseCSS4.rules,
      hasVar = _parseCSS4.hasVar,
      hasOtherMatch = _parseCSS4.hasOtherMatch;

  if (hasApply) {
    rules = replaceApplyWithPlaceholders(rules);
  }
  return { rules: rules, hasInterpolation: hasVar || hasApply || hasOtherMatch };
}